/* 
 * File:   AQService.h
 * Author: matt
 *
 * Created on July 14, 2011, 5:19 PM
 */

#ifndef AQSERVICE_H
#define	AQSERVICE_H

#include <iostream>
#include <boost/thread.hpp>
#include <qpid/messaging/Message.h>
#include "robokind/common/logger.h"

using namespace std;
using namespace qpid::messaging;

namespace robokind{
    
    template<class T>
    class AQService{
        private:
            bool myInitFlag;
            bool myRunFlag;
            bool myPauseFlag;
            boost::thread* myWorkerThread;
            
            virtual void threadSafeWorker() = 0;
            virtual bool _initialize(T* t) = 0;
            virtual void close() = 0;
            
            static void runLoop(AQService* service){
                while(service->myRunFlag){
                    if(service->myPauseFlag){
                        boost::this_thread::sleep(boost::posix_time::milliseconds(10));
                    }else{
                        service->threadSafeWorker();
                    }
                }
                robokind::common::info("Robokind Service Closing");
                service->close();
            }
        public:
            AQService() : myInitFlag(false), myRunFlag(false), myPauseFlag(false){}
            
            virtual void handleCommandMessage(Message &msg) = 0;
            
            void initialize(T* t){
                if(!myInitFlag){
                    robokind::common::info("Service Initializing.");
                    myInitFlag = _initialize(t);
                }
            }
            
			bool isInitialized(){
				return myInitFlag;
			}

			virtual void _start(){}
            void start(){
                if(myInitFlag && !myRunFlag){
                    robokind::common::info("Robokind Service received Start Command");
					_start();
                    myWorkerThread = new boost::thread(AQService::runLoop, this);
                    myRunFlag = true;
                }
            }
            void pause(){
                if(myRunFlag){
                    robokind::common::info("Robokind Service received Pause Command");
                    myPauseFlag = true;
                }
            }
            void resume(){
                if(myRunFlag && myPauseFlag){
                    robokind::common::info("Robokind Service received Resume Command");
                    myPauseFlag = false;
                }
            }
            void stop(){
                if(myRunFlag){
                    robokind::common::info("Robokind Service received Stop Command");
                    myRunFlag = false;
                    myPauseFlag = false;
                }
            }
    };
}
#endif	/* AQSERVICE_H */

